% Generated by roxygen2 (4.0.2): do not edit by hand
\name{analyzeFeatures}
\alias{analyzeFeatures}
\title{Analysis of splice graph features from BAM files}
\usage{
analyzeFeatures(sample_info, which = NULL, features = NULL,
  predict = is.null(features), alpha = 2, psi = 0.1, beta = 0.2,
  gamma = 0.2, min_n_sample = 1, min_overhang = NA, annotation = NULL,
  cores_per_sample = 1, BPPARAM = MulticoreParam(1))
}
\arguments{
\item{sample_info}{\code{data.frame} with sample information including
mandatory character columns \dQuote{sample_name} and \dQuote{file_bam}.}

\item{which}{\code{GRanges} of genomic regions to be considered for
feature prediction, passed to \code{ScanBamParam}}

\item{features}{\code{TxFeatures} or \code{SGFeatures} object}

\item{predict}{Logical indicating whether transcript
features should be predicted from BAM files}

\item{alpha}{Minimum FPKM required for a splice junction to be included}

\item{psi}{Minimum splice frequency required for a splice junction
to be included}

\item{beta}{Minimum relative coverage required for an internal exon
to be included}

\item{gamma}{Minimum relative coverage required for a terminal exon
to be included}

\item{min_n_sample}{Minimum number of samples a feature must be
observed in to be included}

\item{min_overhang}{After merging, all or a subset of terminal exons
sharing a splice site with an internal exon are removed.
\code{min_overhang} specifies the minimum overhang required for a
terminal exon to be included. Use \code{NA} to remove all terminal
exons sharing a splice site with an internal exon. Use \code{NULL}
to disable filtering (not usually recommended, as this will result
in terminal exon predictions for each splice site; disabling filtering
is useful if results are subsequently merged with other predictions
and filtering is postponed until after the merging step).}

\item{annotation}{\code{TxFeatures} object used for annotation}

\item{cores_per_sample}{Number of cores per sample}

\item{BPPARAM}{\code{BiocParallelParam} for processing samples in
parallel, defaults to \code{MulticoreParam(1)}}
}
\value{
\code{SGFeatureCounts} object
}
\description{
High-level function for the prediction and quantification of
splice junctions, exon bins and splice sites from BAM files.
}
\details{
If alignment information is not included in \code{sample_info},
it is obtained directly from BAM files with \code{\link{getBamInfo}}.

Splice junctions and exons are predicted from BAM files with
\code{\link{predictTxFeatures}}.

Known features can be provided as \code{TxFeatures} or
\code{SGFeatures} via argument \code{features}.

If \code{features} is not \code{NULL} and \code{predict} is
\code{TRUE}, known features are augmented with predictions.

Known and/or predicted transcript features are converted to splice
graph features. For details, see \code{\link{convertToSGFeatures}}.

Optionally, splice graph features can be annotated with respect to
a \code{TxFeatures} object provided via argument \code{annotation}.
For details, see the help page for function \code{\link{annotate}}.

Finally, compatible fragment counts for splice graph features are
obtained from BAM files with \code{\link{getSGFeatureCounts}}.
}
\examples{
dir <- system.file("extdata", package = "SGSeq")
si$file_bam <- file.path(dir, "bams", si$file_bam)
sgfc <- analyzeFeatures(si, gr)
}
\author{
Leonard Goldstein
}

